/*******************************************************************************
*    IM-Daemon                                                                 *
*    Copyright (C) 2012-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "incs/constants.h"
#include "incs/logger.h"

char logfile[255];
int loglevel;

char *logLevels[] = {"DEBUG", "INFO", "NOTICE", "ERROR", "FATAL"};
FILE *fpLog;

/**
 * Setze die aktuelle Logstufe
 *
 * @param int level       Logstufe (DEBUG, INFO, usw.)
 */
void setLogLevel(int level)
{
  loglevel = level;
}

/**
 * Setze die aktuelle Logdatei
 *
 * @param char *file      Die Datei, wo die Logevente gespeichert werden
 */
void setLogFile(char *file)
{
  strcpy(logfile, file);
}

/**
 * Initialisiert das Logsystem
 */
void openLogger()
{
  fpLog = fopen(logfile, "at");
  if(fpLog == NULL)
    LOGGER(LOGL_FATAL, "Unable to open log file %s for writing", logfile);
}

/**
 * Schließt das Logsystem
 */
void closeLogger()
{
  fclose(fpLog);
}

/**
 * Loggt auf Datei und, wenn eingestellt, auf der Konsole
 *
 * @param int level       Loglevel (LOGL_DEBUG, LOGL_INFO, usw.)
 * @param char *file      Die Datei, wo diese Meldung generiert worden ist
 * @param int line        Die Zeile, wo diese Meldung generiert worden ist
 * @param char *format    Die Meldung, formatiert wie bei printf
 */
void logger(int level, char *file, int line, const char *format, ...)
{
  va_list ap;
  char *timestr;
  time_t tm;

  if(level < loglevel)
    return;
  tm = time(NULL);
  timestr = ctime(&tm);
  timestr[strlen(timestr) - 1] = '\0';

  if(fpLog != NULL)
  {
    va_start(ap, format);
    fprintf(fpLog, "%s [%s:%d] %s: ", timestr, file, line, logLevels[level]);
    vfprintf(fpLog, format, ap);
    fprintf(fpLog, "\n");
    fflush(fpLog);
    va_end(ap);
  }
  if(level == LOGL_FATAL || loglevel == LOGL_DEBUG)
  {
    va_start(ap, format);
    fprintf(stderr, "%s [%s:%d] %s: ", timestr, file, line, logLevels[level]);
    vfprintf(stderr, format, ap);
    fprintf(stderr, "\n");
    fflush(stderr);
    va_end(ap);
  }

  if(level == LOGL_FATAL)
  {
    closeLogger();
    exit(EXIT_FAILURE);
  }
}
